@@ -5,7 +5,9 @@ __all__ = [
     "InMemoryQueryExecutorBase",
     "FeatureComputeBase",
     "MetricsBase",
-    "AnomaliesBase"
+    "AnomaliesBase",
+    "SyntheticDataGeneratorBase",
+    "DataObserverBase"
 ]
 
 from .fileops import *
@@ -15,3 +17,5 @@ from .notebook_executor import *
 from .feature_compute import *
 from .metrics import *
 from .anomalies import *
+from .synthetic_data_generator import *
+from .observability import *
