@@ -10,7 +10,9 @@ from scipy.spatial import distance
 import seaborn as sns
 import matplotlib.pyplot as plt
 import logging
+
 from enrichsdk.contrib.lib.transforms import note
+from enrichsdk.datasets import DynamicCustomDataset
 
 from .iofn import *
 
@@ -87,6 +89,12 @@ class AnomaliesBase(Compute):
             raise Exception("Specs not defined in profile")
         return profile["specs"]
 
+    def preprocess_spec(self, spec):
+        '''
+        to be overloaded in the derived class
+        '''
+        return spec
+
     def process_spec(self, spec, datasets):
         msg = ""
 
@@ -105,25 +113,44 @@ class AnomaliesBase(Compute):
         name = spec["name"]
         config = spec["config"]
         handlers = self.get_handlers(spec)
+
+        # Name of the dataset to read
         dataset = config["dataset"]
 
-        # first, get the dataset nature
         nature = None
-        paths = datasets[dataset].paths
-        for path in paths:
-            if path['name'] == 'default':
-                nature = path['nature']
-                break
+        datasetobj = None
+        if dataset in datasets:
+            # Clean match
+            datasetobj = datasets[dataset]
+            paths = datasetobj.paths
+            for path in paths:
+                if path['name'] == 'default':
+                    nature = path['nature']
+                    break
+            # Nothing has been found. So choose the first one
+            nature = paths[0]['nature']
+        else:
+            # Match dynamic datasets
+            for dname, dobj in datasets.items():
+                if not isinstance(dobj, DynamicCustomDataset):
+                    continue
+                if dataset.startswith(dobj.name):
+                    nature = dobj.paths[0]['nature']
+                    datasetobj = dobj
+                    break
 
-        if nature == None:
+        if nature is None:
             err = f"No nature found for dataset: {dataset}"
             raise Exception(err)
 
         try:
-            if nature == "db":
+            if isinstance(datasetobj, DynamicCustomDataset):
+                if nature == "s3":
+                    result = self.get_dataset_s3_dynamic(spec, datasetobj)
+            elif nature == "db":
                 result = self.read_db_source(spec)
             elif nature == "s3":
-                result = self.get_dataset_s3(spec, datasets)
+                result = self.get_dataset_s3(spec, datasetobj)
             elif (
                 (generate is not None)
                 and (generate in handlers)
@@ -144,16 +171,15 @@ class AnomaliesBase(Compute):
         strategy = config.get("strategy")
         if strategy == None:
             # We are doing the default anomaly detection which is deviation from mean
-            data, l_msg = self.process_spec_default(result, spec)
+            data = self.process_spec_default(result, spec)
         elif strategy == "custom":
             # Custom callback
-            data, l_msg  = self.process_spec_custom(result, spec)
+            data  = self.process_spec_custom(result, spec)
         elif strategy not in valid_detection_strategies:
             raise Exception(f"{strategy} is not a valid detection strategy")
         else:
             # for now, we fallback to the default strategy
-            data, l_msg  = self.process_spec_default(result, spec)
-        msg += l_msg
+            data  = self.process_spec_default(result, spec)
 
         # The spec processor can return multiple dataframes
         if isinstance(data, pd.DataFrame):
@@ -186,13 +212,32 @@ class AnomaliesBase(Compute):
         # Get hold of the data first...
         sources = self.get_spec_sources(spec, datasets)
 
-        return callback(sources, spec), msg
+        return callback(sources, spec)
 
     def process_spec_default(self, data, spec):
         """
         Handle one specification at a time..
         """
 
+        partialsamplerate = 0.05
+        samplerate_lut = {
+            "all": 1.0,
+            "partial": partialsamplerate,
+            "none": 0.0
+        }
+        tolerances = {
+            "low": 1,
+            "medium": 2,
+            "high": 3,
+        }
+
+        def anomaly_note(row, threshold):
+            distance = row[f"__anomaly_distance__"]
+            if distance > threshold:
+                return f"{(round(distance/threshold,2))}x outside expected sample deviation"
+            return f"within expected sample deviation"
+
+
         msg = ""
         msg += f"Using default centroid distance anomaly detector" + "\n"
 
@@ -201,80 +246,190 @@ class AnomaliesBase(Compute):
 
         # Get hold of the data first...
         name = spec["name"]
-        df = data[name]
-        total_samples = len(df)
+        orig_df = data[name]
+        total_samples = len(orig_df)
+
+        metrics     = config.get("metrics", orig_df.columns)
+        groups      = config.get('groups', [])
+        outputs     = config.get("outputs", [])
+        dimensions  = config.get("dimensions", [])
+        columns     = list(set(metrics + outputs + dimensions))
+
 
-        dimensions = config.get("dimensions", df.columns)
-        outputs = config.get("outputs", [])
-        pkeys = config.get("primarykey", [])
-        columns = list(set(dimensions + outputs + pkeys))
-        df = df[columns]
         msg += f"Combined set of columns: {columns}" + "\n"
+        msg += f"{note(orig_df, 'Original DF')}" + "\n"
 
+        #########
         # default anomaly detection
-        tolerances = {
-            "low": 1,
-            "medium": 2,
-            "high": 3,
-        }
-
-        tolerance = config.get("tolerance", "medium")
+        #########
+        # get tolerance thresold
+        tolerance = config.get("threshold", config.get("thresold", "medium"))
         scalefactor = tolerances.get(tolerance, 2)
 
+        # get the sample strategy for the normal data
+        normal_samples = config.get("normal_samples", "partial")
+        samplerate = samplerate_lut[normal_samples]
+
         msg += f"(tolerance, scalefactor): ({tolerance}, {scalefactor})" + "\n"
 
-        points      = df[dimensions].to_numpy()    # all data as an MxN matrix
-        centroid    = np.mean(points, axis=0)   # the computed centroid of the dataset
-        distances   = distance.cdist(points, np.array([centroid]), 'euclidean') # distances of each point to centroid
-        stddev      = np.std(distances)         # std dev of distances
-        threshold   = stddev*scalefactor
-        anomalies   = np.where(distances.flatten()>threshold, 'anomaly', 'normal')    # flag where anomalies occur
+        logger.debug(f"Setting up for spec: {spec['name']}",
+                         extra={
+                             'transform': self.name,
+                             'data': msg
+                         })
 
-        # add the anomaly label column
-        label = name.replace(' ', '').replace('-', '')
-        label = f"__is_anomaly__"
-        df[label] = pd.Series(anomalies)
+        anomaly_stats = {}
+        plotdata = {}
+        dfs = []
 
-        # sample the normal data if needed
-        normal_samples = config.get("normal_samples", "partial")
-        partialsamplerate = 0.05
-        samplerate_lut = {
-            "all": 1.0,
-            "partial": partialsamplerate,
-            "none": 0.0
-        }
-        samplerate = samplerate_lut[normal_samples]
+        #########
+        # we first do the leaf level, per metric to check for anomalies
+        #########
+        msg = f"Processing metrics: {metrics}" + "\n\n"
 
-        df_a = df[df[label]=='anomaly']
-        n_anomalies   = len(df_a)
+        for metric in metrics:
 
-        df_n = df[df[label]=='normal'].sample(frac=samplerate)
-        df_n = df_n[0:min(3*n_anomalies,len(df_n))] # min 3x n_anomalies or 5% of normal samples
-        n_nsamples    = len(df_n)
+            # make a copy of the df, we'll keep adding anomlay metrics to it
+            df = orig_df[columns].copy()
 
-        df = pd.concat([df_a, df_n])
+            # compute the anomalies for this metric
+            points      = df[metric].to_numpy()     # all data as an MxN matrix
+            centroid    = df[metric].mean()          # the computed centroid of the dataset
+            distances   = abs(points - centroid)    # distances of each point to centroid
+            stddev      = np.nanstd(points)      # std dev of distances
+            threshold   = stddev * scalefactor
+            anomalies   = np.where(distances.flatten()>threshold, 'anomaly', 'normal')    # flag where anomalies occur
+
+            # add columns indicating anomaly label
+            id = f"metric-{metric}"
+            df['id'] = id
+            df['level'] = 'metric'
+            df['name'] = metric
+            df['__is_anomaly__'] = pd.Series(anomalies)
+
+            # add columns indicating reason for anomaly
+            df[f"__anomaly_distance__"] = pd.Series(distances.flatten())
+            df[f"__anomaly_note__"] = df.apply(lambda x: anomaly_note(x, threshold), axis=1)
+
+            df_a = df[df['__is_anomaly__']=='anomaly']
+            n_anomalies = len(df_a)
+            perc_anomalies = round(n_anomalies/total_samples*100, 2)
+
+            df_n = df[df['__is_anomaly__']=='normal'].sample(frac=samplerate)
+            df_n = df_n[0:min(3*n_anomalies,len(df_n))] # min 3x n_anomalies or configured sample of normal samples
+            n_nsamples = len(df_n)
+
+            # for this metric, we now have all the detected anomalies and the sampled normal data
+            sampled_df = pd.concat([df_a, df_n])
+
+            msg += f"--------------------------" + "\n"
+            msg += f"Metric: {metric}" + "\n"
+            msg += f"Computed stddev: {stddev}" + "\n"
+            msg += f"Threshold: {threshold}" + "\n"
+            msg += f"Anomalies: {n_anomalies}/{total_samples}={perc_anomalies}%" + "\n"
+            msg += f"--------------------------" + "\n\n"
+
+            anomaly_stats[id] = {
+                "level": 'metric',
+                "name": metric,
+                "n_anomalies": n_anomalies,
+                "perc_anomalies": perc_anomalies,
+                "n_normalsamples": n_nsamples,
+            }
+            plotdata[id] = df
+
+            dfs.append(sampled_df)
+
+        logger.debug(f"Processed metrics level: {spec['name']}",
+                         extra={
+                             'transform': self.name,
+                             'data': msg
+                         })
 
 
-        # add a column indicating reason for anomaly
-        def anomaly_note(row, threshold):
-            metric = row["__anomaly_metric__"]
-            if metric > threshold:
-                return f"{round(metric/threshold,2)}x outside expected sample deviation"
-            return f"within expected sample deviation"
+        # #########
+        # # then we do the group level, hierarchial
+        # #########
+        msg = f"Processing groups: {groups}" + "\n\n"
 
-        df["__anomaly_metric__"] = pd.Series(distances.flatten())
-        df["__anomay_note__"] = df.apply(lambda x: anomaly_note(x, threshold), axis=1)
+        for group in groups:
+            group_name  = group.get('group')
+            metrics     = group.get('metrics')
 
-        # order the columns, make sure no duplicates
-        cols = pkeys
-        cols += [c for c in df.columns if "__" in c]
-        cols += [c for c in dimensions if c not in cols]
-        cols += [c for c in outputs if c not in cols]
+            # we don't have what we need, skip
+            if group_name == None or metrics == None:
+                continue
+
+            # make a copy of the df, we'll keep adding anomlay metrics to it
+            df = orig_df[columns].copy()
+
+            points      = df[metrics].to_numpy()    # all data as an MxN matrix
+            centroid    = df[metrics].mean().values # the computed centroid of the dataset
+            distances   = distance.cdist(points, np.array([centroid]), 'euclidean') # distances of each point to centroid
+            distances   = np.reshape(distances, len(distances))
+            stddev      = np.nanstd(points)         # std dev of distances
+            threshold   = stddev * scalefactor
+            anomalies   = np.where(distances.flatten()>threshold, 'anomaly', 'normal')    # flag where anomalies occur
+
+            # add columns indicating anomaly label
+            id = f"group-{group_name}"
+            df['id'] = id
+            df['level'] = 'group'
+            df['name'] = group_name
+            df['__is_anomaly__'] = pd.Series(anomalies)
+
+            # add columns indicating reason for anomaly
+            df[f"__anomaly_distance__"] = pd.Series(distances.flatten())
+            df[f"__anomaly_note__"] = df.apply(lambda x: anomaly_note(x, threshold), axis=1)
+
+            df_a = df[df['__is_anomaly__']=='anomaly']
+            n_anomalies = len(df_a)
+            perc_anomalies = round(n_anomalies/total_samples*100, 2)
+
+            df_n = df[df['__is_anomaly__']=='normal'].sample(frac=samplerate)
+            df_n = df_n[0:min(3*n_anomalies,len(df_n))] # min 3x n_anomalies or configured sample of normal samples
+            n_nsamples = len(df_n)
+
+            # for this metric, we now have all the detected anomalies and the sampled normal data
+            sampled_df = pd.concat([df_a, df_n])
+
+            msg += f"--------------------------" + "\n"
+            msg += f"Group: {group_name}" + "\n"
+            msg += f"Computed stddev: {stddev}" + "\n"
+            msg += f"Threshold: {threshold}" + "\n"
+            msg += f"Anomalies: {n_anomalies}/{total_samples}={perc_anomalies}%" + "\n"
+            msg += f"--------------------------" + "\n"
+
+            anomaly_stats[id] = {
+                "level": 'group',
+                "name": group_name,
+                "metrics": metrics,
+                "threshold": threshold,
+                "n_anomalies": n_anomalies,
+                "perc_anomalies": perc_anomalies,
+                "n_normalsamples": n_nsamples,
+            }
+            plotdata[id] = df
+
+            dfs.append(sampled_df)
+
+        logger.debug(f"Processed groups level: {spec['name']}",
+                         extra={
+                             'transform': self.name,
+                             'data': msg
+                         })
+
+        #########
+        # construct the DF for output
+        #########
+        # concat for all metrics
+        df = pd.concat(dfs)
+        # reorder columns
+        first_cols = ['id', 'level', 'name']
+        cols = first_cols + [c for c in df.columns if c not in first_cols]
         df = df[cols]
 
-        perc = round(n_anomalies/total_samples*100, 2)
-        msg += f"Anomalies: {n_anomalies}/{total_samples}={perc}%" + "\n"
-        msg += f"Final columns: {cols}" + "\n"
+        msg = f"Final columns: {df.columns}" + "\n"
 
         window, start_date, end_date = self.get_window_dates(config, self.args)
 
@@ -286,18 +441,26 @@ class AnomaliesBase(Compute):
             "data_end_date": f"{end_date}",
             "strategy": "centroid",
             "tolerance": tolerance,
-            "n_rows": total_samples,
-            "n_anomalies": n_anomalies,
-            "perc_anomalies": perc,
-            "normalsamples": normal_samples,
-            "n_normalsamples": n_nsamples,
             "scalefactor": scalefactor,
-            "threshold": threshold,
+            "normalsamples": normal_samples,
+            "samplerate": samplerate,
+            "n_rows": total_samples,
+            "anomaly_stats": anomaly_stats,
         }
 
         msg += f"Stats: {json.dumps(stats, indent=4)}" + "\n"
 
-        return {name: df, "stats": stats}, msg
+        msg += f"{note(df, 'Anomaly DF')}" + "\n"
+
+        logger.debug(f"Completed spec: {spec['name']}",
+                         extra={
+                             'transform': self.name,
+                             'data': msg
+                         })
+
+        return {name: df, "stats": stats, "plotdata": plotdata}
+
+
 
     def read_db_source(self, source):
 
@@ -413,9 +576,7 @@ class AnomaliesBase(Compute):
             name = dataset.name
             for subset in dataset.subsets:
                 d = f"{name}-{subset['name']}"
-                if d in spec_datasets:
-                    # make a lookup table
-                    datasets[d] = dataset
+                datasets[d] = dataset
 
         return datasets
 
@@ -438,7 +599,42 @@ class AnomaliesBase(Compute):
         return window, start_date, end_date
 
 
-    def get_dataset_s3(self, spec, datasets):
+    def get_dataset_s3_dynamic(self, spec, datasetobj):
+        """
+        Use the dataset object to read the dataset
+        """
+
+        name = spec["name"]
+        config = spec["config"]
+        params = spec.get("params", {})
+
+        # Look for URI and load it
+        uri = config.get('uri', config.get('filename'))
+        filename = uri.replace("s3://","")
+
+        # Load it
+        df = self.read_s3_data(filename, params)
+
+        logger.debug("Read {}".format(name), extra={"transform": self.name})
+
+        # Insert lineage if possible
+        lineage = {
+            "type": "lineage",
+            "transform": self.name,
+            "dependencies": [
+                {
+                    "type": "file",
+                    "nature": "input",
+                    "objects": [uri]
+                },
+            ],
+        }
+
+        self.update_frame(spec, df, lineage)
+
+        return {name: df}
+
+    def get_dataset_s3(self, spec, datasetobj):
         """
         Use the dataset object to read the dataset
         """
@@ -453,17 +649,14 @@ class AnomaliesBase(Compute):
         window, start_date, end_date = self.get_window_dates(config, self.args)
 
         dataset = config["dataset"]
-        pieces = dataset.split('-')
-        dataset_main = "-".join(pieces[:-1])
-        dataset_subset = pieces[-1]
         params = spec.get("params", {})
-
         filename = config.get("filename")
         if filename == None:
             raise Exception(
                 "filename param in config not provided"
             )
 
+        # => Whether to cache
         cache = self.args.get("cache", False)
         cachename = f"{dataset}-{start_date}-{end_date}"
         cachefile = f"cache/{self.name}-cache-" + cachename + ".csv"
@@ -480,8 +673,6 @@ class AnomaliesBase(Compute):
                 df = pd.read_csv(cachefile, **params)
                 return {name: df}
 
-        datasetobj = self.get_dataset(dataset_main)
-
         df, metadata = datasetobj.read_data(
             start_date,
             end_date,
@@ -525,6 +716,7 @@ class AnomaliesBase(Compute):
 
         id          = spec['id']
         name        = spec['name']
+        namespace   = spec.get('namespace', 'default')
         identifier  = self.get_identifier(spec)
         run_date    = self.args['run_date']
         s3          = self.args['s3']
@@ -537,49 +729,67 @@ class AnomaliesBase(Compute):
         df = data[name]
         df["__run_date__"] = run_date
 
-        dimensions  = config.get('dimensions', df.columns)
+        metrics  = config.get('metrics', df.columns)
 
         # get the stats object
         stats = data["stats"]
+        plotdata = data["plotdata"]
 
         # where are we storing it?
-        anomaliesfile   = os.path.join(self.args['s3root'], f"{identifier}/{run_date}/{epoch}__outliers.csv")
-        statsfile       = os.path.join(self.args['s3root'], f"{identifier}/{run_date}/{epoch}__stats.json")
-        plotfile_1dim   = os.path.join(self.args['s3root'], f"{identifier}/{run_date}/{epoch}__plot_1dim.png")
-        plotfile_ndim   = os.path.join(self.args['s3root'], f"{identifier}/{run_date}/{epoch}__plot_ndim.png")
+        targetdir = os.path.join(self.args['s3root'], namespace, f"{identifier}/{run_date}/{epoch}")
+        anomaliesfile   = os.path.join(targetdir, "outliers.csv")
+        statsfile       = os.path.join(targetdir, f"stats.json")
 
-        # plot the results
-        plot = pd.plotting.parallel_coordinates(df,
-                                                class_column='__is_anomaly__',
-                                                cols=dimensions,
-                                                colormap=plt.get_cmap("Set1"))
-        plt.title('Outlier Mapping by Dimension')
-        # plt.savefig("/Users/achintthomas/Desktop/test_ndim.png")        # plots
-        img_data = io.BytesIO()
-        plt.savefig(img_data, format='png')
-        img_data.seek(0)
-        with s3.open(plotfile_ndim, 'wb') as fd:
-            fd.write(img_data.getbuffer())
-        plt.clf()
+        ## plot the results
+        msg = note(df, f"{name} - Anomalies Tagged") + "\n"
+        msg += f"Anomalies: {anomaliesfile}" + "\n"
+        msg += f"Stats: {statsfile}" + "\n"
 
-        df_plot = df[['__is_anomaly__', '__anomaly_metric__']]
-        df_plot['y'] = 0 # for the y-axis
-        plot = sns.scatterplot(data=df_plot, x="__anomaly_metric__", y="y",
-                                hue="__is_anomaly__",
-                                style="__is_anomaly__")
-        plt.axvline(x=stats['threshold'],
-                    ymin=0.3, ymax=0.7,
-                    color='k', linestyle='--')
-        plt.annotate('threshold',
-                        xy=(stats['threshold'], 0.025))
-        plt.title('Outlier Distance from Expected Sample Deviation')
-        #plt.savefig("/Users/achintthomas/Desktop/test_1dim.png")
-        img_data = io.BytesIO()
-        plt.savefig(img_data, format='png')
-        img_data.seek(0)
-        with s3.open(plotfile_1dim, 'wb') as fd:
-            fd.write(img_data.getbuffer())
-        plt.clf()
+        # for each level-name combination (indexed by id)
+        for id in set(df['id'].values):
+
+            plotfile = os.path.join(targetdir, f"plot/{id}.png")
+            msg += f"plot: {plotfile}" + "\n"
+            data_df = plotdata[id]
+
+            # Plot the data
+            if stats['anomaly_stats'][id]['level'] == 'metric':
+                # for the metrics level
+                x = stats['anomaly_stats'][id]['name']
+                plot = sns.displot(data_df,
+                                    x=x,
+                                    hue="__is_anomaly__",
+                                    stat="probability")
+                plot.fig.subplots_adjust(top=.95)
+                plot.set(title=f'Metric Distribution ({id})')
+            else:
+                # for the groups level
+                metrics = stats['anomaly_stats'][id]['metrics']
+                data_df = data_df[metrics+['__is_anomaly__']]  # only retain the columns we need
+                plot = sns.pairplot(data_df,
+                                    hue='__is_anomaly__')
+                plot.fig.subplots_adjust(top=.95)
+                plot.fig.suptitle(f'Pairplot Structure ({id})')
+
+            # save the image data
+            fig = plot.fig
+
+            # save locally when testing
+            if self.args.get('testmode', False):
+                outputpath = self.args['test_outpath']
+                try:
+                    os.makedirs(outputpath)
+                except:
+                    pass
+                testplotfile = os.path.join(outputpath, f'{id}.png')
+                fig.savefig(testplotfile)
+
+            # write to s3
+            img_data = io.BytesIO()
+            fig.savefig(img_data, format='png')
+            img_data.seek(0)
+            with s3.open(plotfile, 'wb') as fd:
+                fd.write(img_data.getbuffer())
 
         # write to s3
         # anomalies
@@ -590,9 +800,6 @@ class AnomaliesBase(Compute):
         with s3.open(statsfile, 'w') as fd:
             json.dump(stats, fd)
 
-        msg = note(df, f"{name} - Anomalies Tagged") + "\n"
-        msg += f"s3 location: {anomaliesfile}" + "\n"
-
         logger.debug("Wrote anomalies".format(name),
                         extra={"transform": self.name,
                                 "data": msg})
@@ -633,6 +840,14 @@ class AnomaliesBase(Compute):
             if not enable:
                 continue
 
+            # pre-process the spec
+            spec = self.preprocess_spec(spec)
+            logger.debug(f"Preproccessed spec: {spec['name']}",
+                             extra={
+                                 'transform': self.name,
+                                 'data': json.dumps(spec, indent=4)
+                             })
+
             # process the spec to detect outliers
             try:
                 data = self.process_spec(spec, datasets)
