@@ -1156,6 +1156,33 @@ class Node(metaclass=NodeMeta):
         """
         return True
 
+    def add_marker(self, state, name=None, suffix="Completed"):
+        """
+        Adds an object to the state to force order. Doesnt
+        serve any other purpose
+
+        Parameters
+        ----------
+        state (obj): State object
+        """
+        detail = {
+            "df": {},
+            "transform": self.name,
+            "frametype": "dict",
+            "params": {},
+            "history": [],
+        }
+
+        # What should I call the state object?
+        if name is None:
+            name = self.name
+            if suffix is not None:
+                name += "_" + suffix
+
+        # Dump it into the shared state
+        state.update_frame(name, detail, create=True)
+
+
     #####################################################
     # => Execute
     #####################################################
@@ -1576,6 +1603,7 @@ class Transform(Node):
             List of dicts. Each dict has 'type' (columns) and
             columns metadata (a dict)
         """
+
         columns = self.get_column_metadata(name, df)
 
         return [{"type": "compute", "columns": columns}]
