@@ -5,6 +5,7 @@ import re
 from datetime import datetime
 import traceback
 import pandas as pd
+from hashlib import sha256
 from s3fs import S3FileSystem
 from ..lib.exceptions import NoDataFound
 
@@ -102,7 +103,7 @@ class DataSource(object):
              "subsets": [
                  {
                      "name": "v1culltable",
-                     "pattern": ".*v1culltable.csv$"
+                     "filename": "v1culltable.csv"
                  },...
               ]
           }
@@ -134,9 +135,11 @@ class DataSource(object):
         if not isinstance(names, list):
             names = [names]
 
-        matches = [((name == self.name) or (name in self.alt_names)) \
-                                   for name in names]
-        return any(matches)
+        allnames = [self.name] + self.alt_names
+        subsets = self.get_subsets()
+        allnames += [self.name + "-" + s for s in subsets]
+
+        return any([name in allnames for name in names])
 
     def set_resolve(self, resolve):
         """
@@ -235,6 +238,31 @@ class DataSource(object):
         """
         return [s["name"] for s in self.subsets]
 
+    def get_subsets_detail(self):
+        """
+        Return the names of subsets available
+        """
+        return self.subsets
+
+    def resolve_subset_name(self, name, params={}):
+        """
+        """
+        if callable(name):
+            return name(self, params)
+
+        return str(name)
+
+
+    def get_subset_detail(self, name):
+        """
+        Return the names of subsets available
+        """
+        for s in self.subsets:
+            if s['name'] == name:
+                return s
+
+        raise Exception("Unknown subset: {}".format(name))
+
     def get_subset_description(self, name):
         """
         Get the description for a subset
@@ -263,11 +291,17 @@ class DataSource(object):
             return False
 
         #
-        if "pattern" not in subset:
+        if (("filename" not in subset) and ("pattern" not in subset)):
             return False
 
+        if "filename" in subset:
+            pattern = subset['filename'] + "$"
+        else:
+            pattern = subset["pattern"]
+
+        # What to check?
         filename = spec["filename"]
-        pattern = subset["pattern"]
+
         return re.search(pattern, filename) is not None
 
     def get_path(self, name, full=False, resolve=None):
@@ -294,6 +328,8 @@ class DataSource(object):
                 )
             if "path" not in elem:
                 raise Exception("paths element should have a 'path'")
+            if "name" not in elem:
+                raise Exception("paths element should have a 'name'")
 
     def __str__(self):
         return "[{}] {}".format(self._type, self.name)
@@ -337,12 +373,12 @@ class Dataset(DataSource):
             "subsets": [
                 {
                     "name": "CatalogScore",
-                    "pattern": ".*catalogscore.csv$",
+                    "filename": "catalogscore.csv",
                     "description": "Score of all products in the catalog"
                 },
                 {
                     "name": "ProductWeight",
-                    "pattern": ".*productweight.csv$",
+                    "filename": "productweight.csv",
                     "description": "Assortment weight for each product"
                 }
             ]
@@ -367,7 +403,12 @@ class Dataset(DataSource):
             }
         """
 
-    def sample(self, filename, safe=True, fd=None, nrows=10):
+        # Insert instance id
+        name = self.name
+        for s in self.subsets:
+            s['instanceid'] = sha256(f"{name}:{s['name']}".encode("utf-8")).hexdigest()[:16]
+
+    def sample(self, filename, safe=True, fd=None, nrows=10, encoding='utf-8'):
         """
         Sample a file belonging to this dataset.Subclass and
         overload this function if the data is sensitive.
@@ -390,13 +431,13 @@ class Dataset(DataSource):
         if filename.lower().endswith(".csv"):
             df = pd.read_csv(fd, nrows=nrows)
         elif filename.lower().endswith(".csv.gz"):
-            df = pd.read_csv(fd, nrows=nrows, compression="gzip", error_bad_lines=False)
+            df = pd.read_csv(fd, nrows=nrows, compression="gzip", error_bad_lines=False, encoding=encoding)
         elif filename.lower().endswith(".tsv.gz"):
             df = pd.read_csv(
-                fd, nrows=nrows, sep="\t", compression="gzip", error_bad_lines=False
+                fd, nrows=nrows, sep="\t", compression="gzip", error_bad_lines=False, encoding=encoding
             )
         elif filename.lower().endswith(".tsv"):
-            df = pd.read_csv(fd, nrows=nrows, sep="\t")
+            df = pd.read_csv(fd, nrows=nrows, sep="\t", encoding=encoding)
         elif filename.lower().endswith(".pq") or filename.lower().endswith(
             ".pq.sample"
         ):
@@ -722,8 +763,8 @@ class Dataset(DataSource):
 
         Args:
              start_date (str): Starting date to scan
-             end_date (str): Ending date of the scane
-             filename (str): Filename within each day's directory
+             end_date (str): Ending date of the scan
+             filename (str): Filename within each day's directory. If None, readfunc will be called with the directory name
              readfunc (method): Callback
              errors (bool): What to do on failure. True = bailout
              name (str): Which subset to look at?
@@ -747,7 +788,15 @@ class Dataset(DataSource):
         # Now compose all the dataframes...
         _dfs = []
         for dt in dates:
-            fullpath = os.path.join(path, dt["name"], filename)
+
+            if dt['name'] not in ['', '.']:
+                fullpath = os.path.join(path, dt["name"])
+            else:
+                fullpath = path
+
+            if filename is not None:
+                fullpath = os.path.join(fullpath, filename)
+
             try:
                 metadata["files"].append(fullpath)
                 df = readfunc(fullpath, params, date=dt["name"])
@@ -774,6 +823,40 @@ class Dataset(DataSource):
 
         return df, metadata
 
+    def read_subset(
+            self,
+            start_date,
+            end_date,
+            subset,
+            readfunc,
+            errors=True,
+            name="default",
+            params={},
+            resolve={},
+    ):
+        """
+        Read a particular subset of the dataset
+        """
+
+        filename = None
+        for s in self.subsets:
+            if ((s['name'] == subset) or
+                ('filename' in s)):
+                filename = s['filename']
+
+        if filename is None:
+            raise Exception(f"Either subset doesnt exist or define a filename: {subset}")
+
+        return self.read_data(
+            start_date,
+            end_date,
+            filename,
+            readfunc,
+            errors=errors,
+            name=name,
+            params=params,
+            resolve=resolve)
+
     def write_data_note(
         self,
             metadata
@@ -785,6 +868,31 @@ class Dataset(DataSource):
             self.registry.access(self, metadata, nature="read")
 
 
+class DynamicCustomDataset(Dataset):
+    """
+    Class to handle dynamic subset resolution based on context
+    """
+
+    def get_subset_description(self, name):
+        """
+        This has been already
+        """
+        return f"Custom dataset named {name}"
+
+    def get_name(self, params):
+        name = params['filename']
+        name = os.path.basename(name)
+        name = name[:name.rindex(".")]
+        name = "".join([x if x.isalnum() else "-" for x in name])
+        return name
+
+    def in_subset(self, name, params):
+        """
+        name is 'possibly' resolved dynamically and may not match the
+        'name' in the subset field
+        """
+        return True
+
 class DatabaseTable(DataSource):
     def __init__(self, params, *args, **kwargs):
         super().__init__(params, *args, **kwargs)
