@@ -55,6 +55,7 @@ __all__ = [
     "print_log",
     "testdata_action",
     "get_sample_siteconf",
+    "get_sample_djangoenv",
     "get_sample_versionmap",
     "populate",
 ]
@@ -1179,6 +1180,35 @@ def get_sample_siteconf():
     """
     return siteconf
 
+def get_sample_djangoenv():
+
+    cwd = os.getcwd()
+
+    if "enrich" not in cwd:
+        cwd += "/enrich"
+    djangoenv = f"""\
+
+# Global settings
+export ENRICH_ROOT="{cwd}"
+export ENRICH_DATA="$ENRICH_ROOT/data"
+export ENRICH_ETC="$ENRICH_ROOT/etc"
+export ENRICH_CUSTOMERS="$ENRICH_ROOT/customers"
+
+export DATABASE_URL="sqlite:///$ENRICH_ROOT/shared/db.sqlite3"
+export DJANGO_SETTINGS_MODULE=config.settings.local
+export DJANGO_DEFAULT_FROM_EMAIL="support@scribbledata.io"
+export DJANGO_SECRET_KEY="hz[+T3&F8?bOM.cTkh2o!Ox#OI,QM8R4bGjR=O4e)@iJ[DV:M-"
+export DJANGO_EXTRA_ALLOWED_HOSTS=""
+export DJANGO_UWSGI_MODE="production"
+export EMAIL_HOST="email-smtp.us-east-1.amazonaws.com"
+export EMAIL_HOST_USER="AKIAZYH5XXX"
+export EMAIL_HOST_PASSWORD="BJGTIp88VjXXX"
+export EMAIL_PORT=587
+export EMAIL_USE_TLS="TRUE"
+
+
+"""
+    return djangoenv
 
 def get_sample_versionmap():
     versionmap = """\
@@ -1269,6 +1299,28 @@ def populate(context={}):
             except:
                 print(redbg("versionmap could not be created"))
 
+    # => djangoenv
+    djangoenv = None
+    if "djangoenv" in os.environ:
+        djangoenv = os.environ["django-env"]
+    elif "djangoenv" in context:
+        djangoenv = context["django-env"]
+    elif "ENRICH_ETC" in os.environ:
+        djangoenv = os.path.join(os.environ["ENRICH_ETC"], "django-env")
+    else:
+        print(redbg("Path for django-env not found"))
+
+    if djangoenv is not None:
+        if os.path.exists(djangoenv):
+            print(orangebg("django-env already present"))
+        else:
+            try:
+                with open(djangoenv, "w") as fd:
+                    fd.write(get_sample_djangoenv())
+                print(greenbg("django-env initialized"))
+            except:
+                print(redbg("django-env could not be created"))
+
     if "ENRICH_CUSTOMERS" in os.environ and "ENRICH_DATA" in os.environ:
         enrich_customers = os.environ["ENRICH_CUSTOMERS"]
         enrich_data = os.environ["ENRICH_DATA"]
@@ -1644,18 +1696,20 @@ def test_create_state(params):
         return nextstep
 
     if hasattr(obj, "testdata"):
-        if ("data" not in obj.testdata) or (not isinstance(obj.testdata["data"], dict)):
-            print(redbg("Module's testdata should have dict 'data' element"))
-            nextstep = False
 
-        if nextstep and len(obj.testdata["data"]) == 0:
-            print(orangebg("Module's testdata usually has non-trivial 'data' element"))
-
-        if nextstep and obj.is_skin():
-            if ("runs" not in obj.testdata) or (len(obj.testdata["runs"]) == 0):
-                print(redbg("Renderer module's testdata should have 'runs' element"))
+        if "data" in obj.testdata:
+            if not isinstance(obj.testdata["data"], dict):
+                print(redbg("Module's testdata should have dict 'data' element"))
                 nextstep = False
 
+            if nextstep and len(obj.testdata["data"]) == 0:
+                print(orangebg("Module's testdata usually has non-trivial 'data' element"))
+
+            if nextstep and obj.is_skin():
+                if ("runs" not in obj.testdata) or (len(obj.testdata["runs"]) == 0):
+                    print(redbg("Renderer module's testdata should have 'runs' element"))
+                    nextstep = False
+
     if not nextstep:
         return nextstep
 
