@@ -92,12 +92,13 @@ def process():
     Getting started:
        version:  Version of this sdk
        start:    First time instructions
-       env:      Setup the environment context
+       env:      Setup/check the setup
 
     \b
     Development:
-       init:  Bootstrap modules including transforms*
-       test:      Test transforms, manage datasets
+       init:   Bootstrap modules including transforms*
+       test:   Test transforms, manage datasets
+       manage: Manage services such as mongo
 
     \b
     Server:
@@ -469,6 +470,15 @@ def _versionmap():
     print(package.get_sample_versionmap())
 
 
+@envsetup.command("dump-versionmap")
+def _dump_versionmap():
+    """
+    Show versionmap
+    """
+
+    timestamp, versionmap = lib.read_versionmap()
+    print(json.dumps(versionmap, indent=4))
+    
 @envsetup.command("sample-siteconf")
 def _siteconf():
     """
@@ -482,6 +492,17 @@ def _siteconf():
     print("----")
     print(package.get_sample_siteconf())
 
+@envsetup.command("sample-djangoenv")
+def _env():
+    """
+    Sample django-env environment variables.
+    """
+
+    print(
+        "Please store this in django-env. Default is $ENRICH_ROOT/etc/django-env"
+    )
+    print(package.get_sample_djangoenv())
+
 
 @envsetup.command("populate")
 @click.option("--context", default=None, help="Context file")
@@ -1401,6 +1422,41 @@ api.add_command(scheduler)
 api.add_command(_app_list)
 api.add_command(_app_detail)
 
+##########################################################
+# Manage commands
+##########################################################
+@click.group("manage", hidden=True)
+@click.option("--context", default=None, help="Environment file")
+@click.pass_context
+def manage(ctx, context):
+    """
+    Manage mongo/postgres etc.
+    """
+
+    # use the context file to set the environment
+    try:
+        context = lib.Context(context)
+    except Exception as e:
+        raise
+
+@manage.command("mongo")
+@click.pass_context
+def mongo_manage(ctx):
+    """
+    Start/stop mongo
+    """
+
+    commands = [
+        "docker run --name alpha -d -p 27017:27017 mongo",
+        "docker container stop alpha",
+        "docker container ls"
+        "docker container prune"
+    ]
+
+    for c in commands:
+        print(c)
+
+
 ###################################
 # Add to process
 ###################################
@@ -1411,6 +1467,7 @@ process.add_command(envsetup)
 process.add_command(api)
 process.add_command(test)
 process.add_command(sample)
+process.add_command(manage)
 process.add_command(_show_log)
 
 
