@@ -4,8 +4,13 @@ class APPNAMEConfig(EnrichAppConfig):
     name = 'APPNAME'
     verbose_name = "APPDESC"
     description = f"APPDESC"
-    status = "stable"
+    tags = []
+    status = "alpha"
     enable = True
     filename = __file__
     multiple = False
     composition = True
+
+    @classmethod
+    def get_readme(cls):
+        return """APPDESC"""
